

namespace IRDB_SDK
{
	using namespace std;

	class CallGraph_t;
	class CallGraphNode_t;

	using CallGraphEdge_t    = pair<CallGraphNode_t*,CallGraphNode_t*>;
	using CallGraphNodeSet_t = set<CallGraphNode_t*>;
	using CallSite_t         = Instruction_t*;
	using CallSiteSet_t      = set<CallSite_t>;


	class CallGraphNode_t
	{
		protected:
			CallGraphNode_t() { }
			CallGraphNode_t(const CallGraphNode_t& copy) = delete;
		public:
			virtual ~CallGraphNode_t() { }

			virtual bool        isHellnode()           const  = 0;
			virtual Function_t* getFunction()          const  = 0;
			virtual void        dump(ostream& os=cout) const  = 0;
	};

	class CallGraph_t
	{
		protected:
			CallGraph_t() {}
			CallGraph_t(const CallGraph_t& copy) = delete;
		public:
			virtual ~CallGraph_t() { }
			
		// accessors

			virtual void  addFile(FileIR_t* const firp) = 0;
			virtual bool  edgeExists(const CallGraphEdge_t& edge)        const = 0;
			virtual const CallGraphNodeSet_t& 
			              getCallersOfNode(CallGraphNode_t* const node)  const = 0;
			virtual const CallGraphNodeSet_t& 
			              getCalleesOfNode(CallGraphNode_t* const node)  const = 0;
			/*
			virtual const CallGraphNodeSet_t& 
				       getAncestors
			               (
				        const Function_t* const fn, 
				        const bool skipHellNode = false
				       )                                            const = 0;
			virtual const  CallSiteSet_t& 
			               getCallSites(CallGraphNode_t* const n1)      const = 0;
 			virtual string getNodeName(const CallGraphNode_t* const n1) const = 0;
			*/
			virtual bool   isReachable
			               (
			                CallGraphNode_t* const from, 
			                CallGraphNode_t* const to, 
			                bool skipHellNode = false
			               )                                            const = 0;
			virtual const  CallGraphNode_t& 
			               getDefaultHellNode()                         const = 0; 
			virtual CallGraphNode_t* 
			               findNode(Function_t* const fn)               const = 0;
		// printer
			virtual void   dump(ostream& fout)                     const = 0;

		// factories
			static unique_ptr<CallGraph_t>  factory(FileIR_t* const firp=nullptr);
	};
	ostream& operator<<(ostream& os, const CallGraph_t& cg);
	ostream& operator<<(ostream& os, const CallGraphNode_t& cgn);

}
