namespace IRDB_SDK
{

	using namespace std;

	class Type_t : virtual public BaseObj_t
	{
		protected:
			Type_t() {}
			Type_t(const Type_t& copy) = delete;
		public:
			virtual ~Type_t() {}

			virtual string     getName()         const = 0;
			virtual IRDBType_t getTypeID()       const = 0;
			virtual bool       isUnknownType()   const = 0;
			virtual bool       isVariadicType()  const = 0;
			virtual bool       isAggregateType() const = 0;
			virtual bool       isFuncType()      const = 0;
			virtual bool       isBasicType()     const = 0;
			virtual bool       isPointerType()   const = 0;
			virtual bool       isNumericType()   const = 0;

			virtual void       setTypeID(IRDBType_t ty)       = 0;
			virtual void       setName(const string& newname) = 0;

	};

	class BasicType_t : virtual public Type_t
	{
		protected:
			BasicType_t() {}	
			BasicType_t(const BasicType_t& copy) = delete;
		public:
			virtual ~BasicType_t() {}

			virtual bool isBasicType()   const = 0;
			virtual bool isNumericType() const = 0;
	};

	class PointerType_t : virtual public Type_t
	{
		protected:
			PointerType_t() {}
			PointerType_t(const PointerType_t& copy) = delete;
		public:
			virtual ~PointerType_t() {}

			virtual bool    isPointerType()   const = 0;
			virtual Type_t* getReferentType() const = 0;

			virtual void setReferentType(Type_t* r) = 0;
	};

	class AggregateType_t : virtual public Type_t
	{
		protected:
			AggregateType_t() { }
			AggregateType_t(const AggregateType_t& copy) = delete;
		public:
			virtual ~AggregateType_t() {}

			virtual bool    isAggregateType()                   const = 0;
			virtual size_t  getNumAggregatedTypes()             const = 0;
			virtual Type_t* getAggregatedType(unsigned int pos) const = 0;

			virtual void addAggregatedType(Type_t *t, int pos) = 0;
	};

	class FuncType_t : virtual public Type_t
	{
		protected:
			FuncType_t() {}
			FuncType_t(const FuncType_t& copy) = delete;
		public:
			virtual ~FuncType_t() {}

			virtual bool             isFuncType()       const = 0;
			virtual Type_t*          getReturnType()    const = 0;
			virtual AggregateType_t* getArgumentsType() const = 0;

			virtual void setReturnType(Type_t *t)             = 0;
			virtual void setArgumentsType(AggregateType_t *t) = 0;

	};
}
